#include "built_in.h"

/*
 * constants :
 *  _FLASH_WRITE_LATCH - flash write latch size
 *  _FLASH_ERASE       - flash erase block size
 * are related to mcu flash memory write and erase block sizes
 * and as such are defined in appropriate flash library file.
 * Be aware that these constants are MCU dependent.
 * Compiler will link them from appropriate flash library,
 * so the user does not have to take care of them.
 *
 */

static char block[64];

// ___Boot_Delay96k  22 Bytes
void Boot_Delay96k()  org 97600
{
 asm {
MOVLW       20
MOVWF       R12, 0
MOVLW       121
MOVWF       R13, 0
Boot_Delay96k:
DECFSZ      R13, 1, 0
BRA         Boot_Delay96k
DECFSZ      R12, 1, 0
BRA         Boot_Delay96k
NOP
NOP
  }
}

// _Susart_Init 24 Bytes
void Susart_Init(unsigned short brg_reg) org 97952 {
  unsigned short i;

  RCSTA = 0x90;     //dactivation du port srie etActive la rception continue
  TXSTA = 0x26;     // Registre de statut et de contrle dmission (USART)
  TRISC.B7 = 1;
  TRISC.B6 = 0;

  while (PIR1.RCIF)   // Registre de signalisation dinterruptions de priphriques
    i = RCREG;        // Registre de rception de lUSART

  SPBRG = brg_reg;      // Registre du Gnrateur de Baud de lUSART
}

//  _Susart_Write 12 Bytes
void Susart_Write(unsigned short data_) org 97904 {

  while (!TXSTA.TRMT)     // bit de statut du registre  dcalage dmission
    ;
  TXREG = data_;          // Registre de transmission de lUSART
}

// _Susart_Data_Ready  10 Bytes
unsigned short Susart_Data_Ready() org 97938 {
  return (PIR1.RCIF);       // Registre de signalisation dinterruptions de priphriques
}

// _Susart_Read  18 Bytes
unsigned short Susart_Read() org 97868 {
  unsigned short rslt;
  rslt = RCREG;       // Registre de rception de lUSART

  if (RCSTA.OERR) {
  // Registre de statut et de contrle de rception (USART) OERR : bit dindication de dpassement,
  // 1 = Erreur de dpassement (peut tre effac en effaant le bit CREN)
     RCSTA.CREN = 0;   // 1 = Active la rception continue
     RCSTA.CREN = 1;   // 0 = Dsactive la rception continue
  }
  return rslt;
}

//-------------- Empty function, this is where the start address of 'real'
//               program will be placed (by Write_Begin()). That is why
//               this function must have at least 4 bytes of ROM available
//               after its org address.
// _Start_Program 2 Bytes
void Start_Program() org 0x17FC0 {
}

// _Flash_Write_Sector 106 Bytes
void Flash_Write_Sector(long address, char *sdata) org 97270 {
  unsigned short saveintcon, i, j;

  saveintcon = INTCON;

  //--- erase memory
  TBLPTRL = Lo(address);
  TBLPTRH = Hi(address);
  TBLPTRU = Higher(address);
  //--- required erase sequence
  EECON1.EEPGD = 1;
  EECON1.CFGS = 0;
  EECON1.WREN = 1;
  EECON1.FREE = 1;
  INTCON.GIE = 0;
  EECON2 = 0x55;
  EECON2 = 0xAA;
  EECON1.WR = 1;
  INTCON.GIE = 1;
  asm TBLRD*- ;
  //--- write memory
  FSR0L = Lo(sdata);
  FSR0H = Hi(sdata);
  j = 0;
  while (j < _FLASH_ERASE/_FLASH_WRITE_LATCH) {
    i = 0;
    while (i < _FLASH_WRITE_LATCH) {
      TABLAT  = POSTINC0;
      asm {
        TBLWT+*
      }
      i++;
    }
    EECON1.EEPGD = 1;
    EECON1.CFGS = 0;
    EECON1.WREN = 1;
    INTCON.GIE = 0;
    EECON2 = 0x55;
    EECON2 = 0xAA;
    EECON1.WR = 1;
    j++;
  }
  INTCON.GIE = 1;
  EECON1.WREN = 0;
  //--- restore interrupt
  INTCON = saveintcon;
}

// _Susart_Write_Loop 42 Bytes
unsigned short Susart_Write_Loop(char send, char receive) org 97194 {
  unsigned short rslt = 0;

  LBL_BOOT18_96:
    Boot_Delay96k();
    Susart_Write(send);
    Boot_Delay96k();

    rslt++;
    if (rslt == 255u)
      return 0;
    if (Susart_Read() == receive)
      return 1;
  goto LBL_BOOT18_96;
}

//-------------- This void will recover the bootlocation 0x0000, 0x0001 and
//               0x0002 to point to the bootloaer's main. It will also move
//               the reset vector of the program that is uploaded to a new
//               location, in this case 97656
// _Write_Begin 42 Bytes
void Write_Begin() org 97708 {

  Flash_Write_Sector(0x17FC0, block);
  //--- goto main 97656
  block[0] = 0xBC;
  block[1] = 0xEF;
  block[2] = 0xBE;
  block[3] = 0xF0;

}

//-------------- Starts with bootload
// _Start_Bootload 156 Bytes
void Start_Bootload() org 97996 {
  unsigned short i = 0, xx, yy;
  long j = 0;

  while (1) {
    if (i == 64u) {
      //--- If 32 words (64 bytes) recieved then write to flash
      if (!j)
        Write_Begin();
      Flash_Write_Sector(j, block);

      i = 0;
      j += 0x40;
    }
    //--- Ask for yy
    Susart_Write('y');
    while (!Susart_Data_Ready()) ;
    //--- Read yy
    yy = Susart_Read();
    //--- Ask for xx
    Susart_Write('x');
    while (!Susart_Data_Ready()) ;
    //--- Read xx
    xx = Susart_Read();
    //--- Save xxyy in block[i]
    block[i++] = yy;
    block[i++] = xx;
  }
}
