/*
 * Project name:
     Boot_Test (using mikroC Bootloader on PIC18's 96K ROM)
 * Copyright:
     (c) FOURNET Olivier 05/08/2017
 * Revision History:
     20051120:
       - initial release (for 64K ROM)
	 20170805 :
	   - adaptation du code initial de Mikroelektronika à une version PIC18F de 96K ROM
	 
 * Description:
     This is a serial port Bootloader implementation. All required bootloader
     routines are placed in the boot18_64K file, the only thing that is
     user-configurable is the baudrate of the UART communication, which is set
     in the main() through appropriate call to Susart_Init(brg_reg). The
     USART module is being initialized like this:
       - TXSTA.BRGH = 1 (High-speed mode);
       - SPBRG = brg_reg;
     You can use the appropriate formulas and tables from the target MCU's
     datasheet to establish the desired baudrate.
     Once started in PIC, the bootloader waits for a while (approx. 5 seconds)
     to establish UART communication with the MikroBootloader application on the
     PC. If it fails to do so, the bootloader starts the programme already
     loaded in the MCU (initially it is just a NOP). If the communication is
     established, the bootloader receives the hex file from the PC and places it
     where requred (the MikroBootloader application reads out the hex file and
     takes care of the location where hex is to be placed). When hex download
     has finished, the user is notified by the MikroBootloader to reset the MCU,
     and MCU enters the described bootload sequence again.
 * Test configuration:
     MCU:             PIC18F4685
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39761c.pdf
     Oscillator:      HS, 12.0000 MHz
     Ext. Modules:    -
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/mikroc/pic/
 * NOTES:
     - It's recommended not to alter the start address of the main().
       Otherwise, less space in ROM will be available for the application being bootloaded.
     - RX and TX UART switches should be turned On (board specific).
 */

//-------------- main for 96K PIC18's

char uart_rd;

void main() {
  //ANSEL  = 0;                     // Configure AN pins as digital
  //ANSELH = 0;

  UART1_Init(9600);               // Initialize UART module at 9600 bps
  Delay_ms(100);                  // Wait for UART module to stabilize

  UART1_Write_Text("Start");
  UART1_Write(10);
  UART1_Write(13);

  while (1) {                     // Endless loop
    if (UART1_Data_Ready()) {     // If data is received,
      uart_rd = UART1_Read();     // read the received data,
      UART1_Write(uart_rd);       // and send data via UART
    }
  }
}

